#include "Network.h"

NeuralNetwork::NeuralNetwork(int nI, int nH, int nO) {
	numInputs = nI;
	numHidden = nH;
	numOutputs = nO;
	inputs.resize(numInputs);
	hidden.resize(numHidden);
	outputs.resize(numOutputs);

	// Initialize weights to random values
	weightsInputHidden.resize(numInputs);
	for (int i = 0; i < numInputs; i++) {
		weightsInputHidden[i].resize(numHidden);
		for (int j = 0; j < numHidden; j++) {
			weightsInputHidden[i][j] = ((double)rand() / RAND_MAX) - 0.5;
		}
	}
	weightsHiddenOutput.resize(numHidden);
	for (int i = 0; i < numHidden; i++) {
		weightsHiddenOutput[i].resize(numOutputs);
		for (int j = 0; j < numOutputs; j++) {
			weightsHiddenOutput[i][j] = ((double)rand() / RAND_MAX) - 0.5;
		}
	}
}

void NeuralNetwork::feedForward(std::vector<double>& inputs) {
	this->inputs = inputs;

	// Calculate the activations of the hidden layer
	for (int i = 0; i < numHidden; i++) {
		double sum = 0;
		for (int j = 0; j < numInputs; j++) {
			sum += inputs[j] * weightsInputHidden[j][i];
		}
		hidden[i] = sigmoid(sum);
	}

	// Calculate the activations of the output layer
	for (int i = 0; i < numOutputs; i++) {
		double sum = 0;
		for (int j = 0; j < numHidden; j++) {
			sum += hidden[j] * weightsHiddenOutput[j][i];
		}
		outputs[i] = sigmoid(sum);
	}
}

void NeuralNetwork::backPropagate(std::vector<double>& targets) {
	// Calculate the error for the output layer
	std::vector<double> outputErrors(numOutputs);
	for (int i = 0; i < numOutputs; i++) {
		double error = targets[i] - outputs[i];
		outputErrors[i] = error * sigmoidDerivative(outputs[i]);
	}

	// Calculate the error for the hidden layer
	std::vector<double> hiddenErrors(numHidden);
	for (int i = 0; i < numHidden; i++) {
		double sum = 0;
		for (int j = 0; j < numOutputs; j++) {
			sum += weightsHiddenOutput[i][j] * outputErrors[j];
		}
		hiddenErrors[i] = sum * sigmoidDerivative(hidden[i]);
	}

	// Update the weights between the hidden and output layers
	for (int i = 0; i < numHidden; i++) {
		for (int j = 0; j < numOutputs; j++) {
			weightsHiddenOutput[i][j] += 0.5 * outputErrors[j] * hidden[i];
		}
	}

	// Update the weights between the input and hidden layers
	for (int i = 0; i < numInputs; i++) {
		for (int j = 0; j < numHidden; j++) {
			weightsInputHidden[i][j] += 0.5 * hiddenErrors[j] * inputs[i];
		}
	}
}

std::vector<double> NeuralNetwork::getOutput() {
	return outputs;
}

double NeuralNetwork::sigmoid(double x) {
	return 1.0 / (1.0 + exp(-x));
}

double NeuralNetwork::sigmoidDerivative(double x) {
	return x * (1 - x);
}